(*
 *                                                                                             .
 * Module name: CmdLine_Utilities.pas; Module type: Delphi unit; Language: Delphi 5            .
 *                                                                                             .
 * Author: Marcus F. Nefzger AKA MNe (web:<http://www.nefzger.at/>; mailto:<MNe@nefzger.at>)   .
 * File created: Fri., 01.11.2002; Last modified: Fri., 15.07.2005                             .
 *                                                                                             .
 * Bug report: Report all bugs, wishes, suggestions etc. to <BugReport@nefzger.at>.            .
 *                                                                                             .
 *  Purpose:                                                                                   .
 * ----------                                                                                  .
 *                                                                                             .
 *)

unit
   CmdLine_Utilities;

{$INCLUDE '..\includes\Globals.inc'}

// ----------                                                                                  .
   interface
// ----------                                                                                  .

const
   CMDLINE_VALID    =    0;
   SWITCH_NOT_FOUND = (-1);
   PARAM_NOT_FOUND  =   '';

type
   TSwitchChars = set of Char;

   TCheckResult = record
      crResult     : Integer;
      crValidCount : Integer;
   end;

function CheckCmdLineSwitches (const ASwitches: array of String;
                               const ASwitchChars: TSwitchChars;
                               const AParamRequired: array of Boolean;
                               const ADoubleSwitchChar: Boolean{$IFDEF DelphiPascalCompiler} = False{$ENDIF}): TCheckResult;
function CmdLineSwitch (const ASwitch: String;
                        const ASwitchChars: TSwitchChars;
                        const AParamRequired: Boolean;
                        const ADoubleSwitchChar: Boolean{$IFDEF DelphiPascalCompiler} = False{$ENDIF}): Integer;
function GetFirstInvalidSwitch (const ACheckResult1: TCheckResult;
                                const ACheckResult2: TCheckResult): Integer;
function ParamBySwitch (const ASwitch: String;
                        const ASwitchChars: TSwitchChars;
                     // 18.03.2005: UNNOTWENDIG ?? Ich *VERLANGE* ja einen Parameter!          .
                     // const AParamRequired: Boolean;                                         .
                     // --------------------------------------------------------------------   .
                        const ADoubleSwitchChar: Boolean{$IFDEF DelphiPascalCompiler} = False{$ENDIF}): String;
function ParamBySwitchPosition (const ASwitch: String;
                                const APosition: Integer;
                                const ADoubleSwitchChar: Boolean{$IFDEF DelphiPascalCompiler} = False{$ENDIF}): String;

{$IFDEF CONSOLE}
procedure dbg_PrintParams ({keine Parameter});
{$ENDIF}

// ---------------                                                                             .
   implementation
// ---------------                                                                             .

uses
   SysUtils;

const
   DEMAND_PARAM_SEPARATION = TRUE;

   CPY_OFFSET: packed array [Boolean] of Integer
      = (2,3);

//  Input:                                                                                     .
// --------                                                                                    .
// ASwitches         : Array of supported command line switches, eg. ['help','verbose'],       .
//                     without (!) a leading switch char.                                      .
// ASwitchChars      : Set of supportet leading switch chars, eg. ['-','/'].                   .
// AParamRequired    : True, if one ore more parameters can/ may be concated to one or         .
//                     more switches, False otherwise.                                         .
// ADoubleSwitchChar : True, if doubling the switch char(s) is necessary/ supported,           .
//                     eg. '--', False otherwise.                                              .
//  Result:                                                                                    .
// ---------                                                                                   .
// .crResult     : CMDLINE_VALID on success; the position of the first unknown command         .
//                 line switch on error.                                                       .
// .crValidCount : Number of command line switches which are valid (may be zero, an may        .
//                 be up to ParamCount).                                                       .
// ----------------------------------------------------------------------------                .
function CheckCmdLineSwitches (const ASwitches: array of String;
                               const ASwitchChars: TSwitchChars;
                               const AParamRequired: array of Boolean;
                               const ADoubleSwitchChar: Boolean{$IFDEF DelphiPascalCompiler} = False{$ENDIF}): TCheckResult;
const
   PARAM_NA      = 0;
   PARAM_INVALID = 1;
   PARAM_VALID   = 2;
var
   v_map: packed array [Byte] of Byte;
   i, pos: Integer;
   s: String;
begin
   Result.crResult := CMDLINE_VALID;
   Result.crValidCount := 0;
   for i := 0 to High(v_map) do
      v_map[i] := PARAM_NA;           // Schalter an dieser Position ist 'uninteressant'       .
   for i := 1 to ParamCount do begin
      s := ParamStr(i);
      if (   (ADoubleSwitchChar) and ((s[1] in ASwitchChars) and    (s[2] in ASwitchChars)) and (s[1] = s[2])) or
         (not(ADoubleSwitchChar) and ((s[1] in ASwitchChars) and not(s[2] in ASwitchChars))                  ) then
         v_map[i - 1] := PARAM_INVALID; // fr alle Parameter: auf 'ungltig' setzen           .
   end;
   for i := 0 to High(ASwitches) do begin
      pos := CmdlineSwitch(ASwitches[i],ASwitchChars,AParamRequired[i],ADoubleSwitchChar);
      if (pos <> SWITCH_NOT_FOUND) then
         v_map[pos - 1] := PARAM_VALID;
   end;
   for i := ParamCount downto 1 do  // ber _ALLE_ Parameter absteigend iterieren:             .
      if ((v_map[i - 1]) = PARAM_VALID) then
         Inc(Result.crValidCount)   // gltiger Parameter? Gltigkeitszhler ++                .
      else if ((v_map[i - 1]) = PARAM_INVALID) then
         Result.crResult := i;      // ungltig: 'ersten' falschen Parameter merken            .
end;

// Note(s):                                                                                    .
//  . Set AParameterConcated to True to allow params in the form of '-yFOO';                   .
//    set to False to force the usage of '-y FOO'. Whatever you need ;)                        .
//  . ASwitchChars may not be [] !                                                             .
// ----------------------------------------------------------------------------                .
function CmdLineSwitch (const ASwitch: String;
                        const ASwitchChars: TSwitchChars;
                        const AParamRequired: Boolean;
                        const ADoubleSwitchChar: Boolean{$IFDEF DelphiPascalCompiler} = False{$ENDIF}): Integer;
var
   i: Integer;
   s_ref, s: String;
begin
   Result := SWITCH_NOT_FOUND;
   if (ASwitchChars <> []) then begin
      s_ref := AnsiLowerCase(ASwitch);
      for i := 1 to ParamCount do begin
         s := AnsiLowerCase(ParamStr(i));
         if (s[1] in ASwitchChars) and
            (not(ADoubleSwitchChar) or (s[2] in ASwitchChars)) then
            if ((AParamRequired) and (Pos(s_ref,Copy(s,CPY_OFFSET[ADoubleSwitchChar],
                                                       (Length(s) - CPY_OFFSET[ADoubleSwitchChar] + 1))) = 1)) or
               (s_ref = Copy(s,CPY_OFFSET[ADoubleSwitchChar],
                               (Length(s) - CPY_OFFSET[ADoubleSwitchChar] + 1))) then begin
               Result := i;
               Break;
            end;
      end;
   end;
end;

// ----------------------------------------------------------------------------                .
function GetFirstInvalidSwitch (const ACheckResult1: TCheckResult;
                                const ACheckResult2: TCheckResult): Integer;
begin
   if ((ACheckResult1.crResult = CMDLINE_VALID) and
       (ACheckResult2.crResult = CMDLINE_VALID)) then
      Result := CMDLINE_VALID           // _BEIDE_ Ergebnisse sind ok                          .
   else if (ACheckResult2.crResult = CMDLINE_VALID) then
      Result := ACheckResult1.crResult  // Ergebnis #2 ok? -> Fehler in #1                     .
   else
      Result := ACheckResult2.crResult  // sonst: Ergebnis #1 ok -> Fehler in #2               .
end;

// ----------------------------------------------------------------------------                .
function ParamBySwitch (const ASwitch: String;
                        const ASwitchChars: TSwitchChars;
                     // 18.03.2005: UNNOTWENDIG ?? Ich *VERLANGE* ja einen Parameter!          .
                     // const AParamRequired: Boolean;                                         .
                     // --------------------------------------------------------------------   .
                        const ADoubleSwitchChar: Boolean{$IFDEF DelphiPascalCompiler} = False{$ENDIF}): String;
var
   i: Integer;
begin
   i := CmdLineSwitch(ASwitch,ASwitchChars,True,ADoubleSwitchChar);
   if (i > 0) then
      Result := ParamBySwitchPosition(ASwitch,i,ADoubleSwitchChar)
   else
      Result := PARAM_NOT_FOUND;
end;

// ----------------------------------------------------------------------------                .
function ParamBySwitchPosition (const ASwitch: String;
                                const APosition: Integer;
                                const ADoubleSwitchChar: Boolean{$IFDEF DelphiPascalCompiler} = False{$ENDIF}): String;
var
   s: String;
begin
   if ((APosition >= 1) and (APosition <= ParamCount)) then begin
      s := ParamStr(APosition);
      if (AnsiLowerCase(ASwitch) = AnsiLowerCase(Copy(s,CPY_OFFSET[ADoubleSwitchChar],
                                                        (Length(s) - CPY_OFFSET[ADoubleSwitchChar] + 1)))) then begin // '-y FOO'
         if ((APosition + 1) <= ParamCount) then
            Result := ParamStr(APosition + 1)
         else
            Result := PARAM_NOT_FOUND;
      end else if not(DEMAND_PARAM_SEPARATION) then // '-yFOO'
         Result := Copy(s,(Length(ASwitch) + CPY_OFFSET[ADoubleSwitchChar]),
                          (Length(s) - (Length(ASwitch) + (CPY_OFFSET[ADoubleSwitchChar] - 1))))
      else
         Result := PARAM_NOT_FOUND;
   end else
      Result := PARAM_NOT_FOUND;
end;

{$IFDEF CONSOLE}
// ----------------------------------------------------------------------------                .
procedure dbg_PrintParams ();
var
   i: Integer;
begin
   WriteLn('ParamCount(): ',ParamCount);
   for i := 0 to ParamCount do
      WriteLn(']',ParamStr(i),'[');
end;
{$ENDIF}

   // keine Initialisierung.                                                                   .
end.

